@@ -1,21 +1,24 @@
 from __future__ import absolute_import
 # Copyright (c) 2010-2015 openpyxl
 
-from openpyxl.descriptors import Set, Bool, MinMax, Min, Alias
+from openpyxl.compat import safe_string
+
+from openpyxl.descriptors import Bool, MinMax, Min, Alias, NoneSet
 
 from .hashable import HashableObject
 
 horizontal_alignments = (
     "general", "left", "center", "right", "fill", "justify", "centerContinuous",
-    "distributed", None
-)
+    "distributed", )
 vertical_aligments = (
-    "top", "center", "bottom", "justify", "distributed", None
+    "top", "center", "bottom", "justify", "distributed",
 )
 
 class Alignment(HashableObject):
     """Alignment options for use in styles."""
 
+    tagname = "alignment"
+
     __fields__ = ('horizontal',
                   'vertical',
                   'textRotation',
@@ -26,23 +29,23 @@ class Alignment(HashableObject):
                   'justifyLastLine',
                   'readingOrder',
                   )
-    horizontal = Set(values=horizontal_alignments)
-    vertical = Set(values=vertical_aligments)
-    textRotation = Set(values=range(181))
+    horizontal = NoneSet(values=horizontal_alignments)
+    vertical = NoneSet(values=vertical_aligments)
+    textRotation = NoneSet(values=range(181))
     textRotation.values.add(255)
     text_rotation = Alias('textRotation')
-    wrapText = Bool()
+    wrapText = Bool(allow_none=True)
     wrap_text = Alias('wrapText')
-    shrinkToFit = Bool()
+    shrinkToFit = Bool(allow_none=True)
     shrink_to_fit = Alias('shrinkToFit')
     indent = Min(min=0)
     relativeIndent = Min(min=0)
-    justifyLastLine = Bool()
+    justifyLastLine = Bool(allow_none=True)
     readingOrder = Min(min=0)
 
-    def __init__(self, horizontal='general', vertical='bottom',
-                 textRotation=0, wrapText=False, shrinkToFit=False, indent=0, relativeIndent=0,
-                 justifyLastLine=False, readingOrder=0, text_rotation=None,
+    def __init__(self, horizontal=None, vertical=None,
+                 textRotation=0, wrapText=None, shrinkToFit=None, indent=0, relativeIndent=0,
+                 justifyLastLine=None, readingOrder=0, text_rotation=None,
                  wrap_text=None, shrink_to_fit=None) :
         self.horizontal = horizontal
         self.vertical = vertical
@@ -60,3 +63,10 @@ class Alignment(HashableObject):
         if shrink_to_fit is not None:
             shrinkToFit = shrink_to_fit
         self.shrinkToFit = shrinkToFit
+
+
+    def __iter__(self):
+        for attr in self.__attrs__:
+            value = getattr(self, attr)
+            if value is not None and value != 0:
+                yield attr, safe_string(value)
