@@ -7,52 +7,24 @@ from __future__ import absolute_import
 __docformat__ = "restructuredtext en"
 
 # Python stdlib imports
-import datetime
 import threading
 
 # package imports
-from openpyxl.collections import IndexedList
+from openpyxl.utils.indexed_list import IndexedList
+from openpyxl.utils.datetime  import CALENDAR_WINDOWS_1900
 from openpyxl.worksheet import Worksheet
 from openpyxl.writer.dump_worksheet import DumpWorksheet, save_dump
 from . names.named_range import NamedRange
 from openpyxl.styles import Style
+from openpyxl.styles.styleable import StyleId
+from openpyxl.styles.numbers import BUILTIN_FORMATS
 from openpyxl.writer.excel import save_workbook
-from openpyxl.exceptions import ReadOnlyWorkbookException
-from openpyxl.date_time import CALENDAR_WINDOWS_1900
+from openpyxl.utils.exceptions import ReadOnlyWorkbookException
 from openpyxl.xml import LXML
 from openpyxl.xml.functions import fromstring
 from openpyxl.xml.constants import SHEET_MAIN_NS
 from openpyxl.compat import deprecated
-
-if LXML:
-    from openpyxl.writer.dump_lxml import LXMLWorksheet as DumpWorksheet
-
-
-class DocumentProperties(object):
-    """High-level properties of the document."""
-
-    def __init__(self):
-        self.creator = 'Unknown'
-        self.last_modified_by = self.creator
-        self.modified = self.created = datetime.datetime.now()
-        self.title = 'Untitled'
-        self.subject = ''
-        self.description = ''
-        self.keywords = ''
-        self.category = ''
-        self.company = 'Microsoft Corporation'
-        self.excel_base_date = CALENDAR_WINDOWS_1900
-
-
-class DocumentSecurity(object):
-    """Security information about the document."""
-
-    def __init__(self):
-        self.lock_revision = False
-        self.lock_structure = False
-        self.lock_windows = False
-        self.revision_password = ''
-        self.workbook_password = ''
+from . properties import DocumentProperties, DocumentSecurity
 
 
 class Workbook(object):
@@ -79,23 +51,54 @@ class Workbook(object):
         self.__read_only = read_only
         self.__thread_local_data = threading.local()
         self.shared_strings = IndexedList()
-        self.shared_styles = IndexedList()
-        self.shared_styles.add(Style())
+
+        self._setup_styles()
         self.loaded_theme = None
         self._worksheet_class = worksheet_class
         self.vba_archive = None
-        self.style_properties = None
+        self.is_template = False
+        self._differential_styles = []
         self._guess_types = guess_types
         self.data_only = data_only
         self.relationships = []
         self.drawings = []
-        self.code_name = u'ThisWorkbook'
-
+        self.code_name = None
+        self.excel_base_date = CALENDAR_WINDOWS_1900
         self.encoding = encoding
 
         if not self.write_only:
             self.worksheets.append(self._worksheet_class(parent_workbook=self))
 
+
+    def _setup_styles(self):
+        """Bootstrap styles"""
+        from openpyxl.styles.alignment import Alignment
+        from openpyxl.styles.borders import DEFAULT_BORDER
+        from openpyxl.styles.fills import DEFAULT_EMPTY_FILL, DEFAULT_GRAY_FILL
+        from openpyxl.styles.fonts import DEFAULT_FONT
+        from openpyxl.styles.protection import Protection
+        from openpyxl.styles.colors import COLOR_INDEX
+
+        self._fonts = IndexedList()
+        self._fonts.add(DEFAULT_FONT)
+
+        self._alignments = IndexedList([Alignment()])
+
+        self._borders = IndexedList()
+        self._borders.add(DEFAULT_BORDER)
+
+        self._fills = IndexedList()
+        self._fills.add(DEFAULT_EMPTY_FILL)
+        self._fills.add(DEFAULT_GRAY_FILL)
+
+        self._number_formats = IndexedList()
+
+        self._protections = IndexedList([Protection()])
+
+        self._colors = COLOR_INDEX
+        self._cell_styles = IndexedList([StyleId()])
+
+
     @deprecated('this method is private and should not be called directly')
     def read_workbook_settings(self, xml_source):
         self._read_workbook_settings(xml_source)
@@ -103,20 +106,37 @@ class Workbook(object):
     def _read_workbook_settings(self, xml_source):
         root = fromstring(xml_source)
         view = root.find('*/' '{%s}workbookView' % SHEET_MAIN_NS)
-        if view is None:
-            return
+        if view is not None:
+            if 'activeTab' in view.attrib:
+                self.active = int(view.attrib['activeTab'])
+
+    @property
+    def shared_styles(self):
+        """
+        Legacy
+        On the fly conversion of style references to style objects
+        """
+        styles = []
+        for sid in self._cell_styles:
+            font = self._fonts[sid.font]
+            fill = self._fills[sid.fill]
+            border = self._borders[sid.fill]
+            alignment = self._alignments[sid.alignment]
+            protection = self._protections[sid.protection]
+            nf_id = sid.number_format
+            if nf_id < 164:
+                number_format = BUILTIN_FORMATS.get(nf_id, "General")
+            else:
+                number_format = self._number_formats[sid.number_format - 164]
+            styles.append(Style(font, fill, border, alignment,
+                                number_format, protection))
+            return styles
 
-        if 'activeTab' in view.attrib:
-            self.active = int(view.attrib['activeTab'])
 
     @property
     def _local_data(self):
         return self.__thread_local_data
 
-    @property
-    def excel_base_date(self):
-        return self.properties.excel_base_date
-
     @property
     def read_only(self):
         return self.__read_only
@@ -187,16 +207,11 @@ class Workbook(object):
     def get_sheet_by_name(self, name):
         """Returns a worksheet by its name.
 
-        Returns None if no worksheet has the name specified.
-
         :param name: the name of the worksheet to look for
         :type name: string
 
         """
-        try:
-            return self[name]
-        except KeyError:
-            return
+        return self[name]
 
     def __contains__(self, key):
         return key in set(self.sheetnames)
